/*
   PhysicsThread Project

      Copyright (C) 2008 Bepu Entertainment LLC

      This demonstration software, excluding the BEPUphysics.dll and BEPUphysics.xml (which are handled independently by their accompanying license),
      is provided 'as-is', without any express or implied warranty.  In no event will the authors be held liable for any damages
      arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
    Ross Nordby
    norbo@bepu-games.com
    Harrison Nordby
    squashwell@bepu-games.com
 */

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Threading;
using BEPUphysics;
using System.Diagnostics;

namespace PhysicsThread
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class ThreadingGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Space space;

        Model cubeModel;
        public Camera camera;

        Thread thread;

        public ThreadingGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            
            space = new Space();
            space.simulationSettings.useInternalTimeStepping = true; //This will let the space step forward varying amounts of time per update.
            space.simulationSettings.gravity = new Vector3(0, -9.81f, 0);
            
            camera = new Camera(new Vector3(0, 0, 10), 10f, 0, 0, Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(33.75f * (float)graphics.GraphicsDevice.Viewport.Width / graphics.GraphicsDevice.Viewport.Height), (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height, .1f, 10000.0f), this);
            Components.Add(camera);
            
            //Set up the physics thread and start it.
            thread = new Thread(physicsLoop);
            thread.IsBackground = true;
            thread.Start();


            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            cubeModel = Content.Load<Model>("cube");

            Box box = new Box(new Vector3(0, -4, 0), 40, 1, 40);
            space.add(box);
            EntityModel entityModel = new EntityModel(box, cubeModel, Matrix.CreateScale(box.width, box.height, box.length), this);
            Components.Add(entityModel);

            Random random = new Random();
            for (int k = 0; k < 50; k++)
            {
                box = new Box(new Vector3((float)random.NextDouble() * 10 - 5, (float)random.NextDouble() * 10, (float)random.NextDouble() * 10 - 5), 1, 1, 1, 10);
                space.add(box);
                entityModel = new EntityModel(box, cubeModel, Matrix.CreateScale(box.width, box.height, box.length), this);
                Components.Add(entityModel);
            }


            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            // TODO: Add your update logic here
            
            base.Update(gameTime);
        }

        /// <summary>
        /// Updates the physics using a high frequency timer.
        /// </summary>
        void physicsLoop()
        {
            double dt;
            double time;
#if XBOX360
            thread.SetProcessorAffinity(5);
#endif
            double previousTime = Stopwatch.GetTimestamp() / Stopwatch.Frequency;
            

            while (true)
            {
               
                time = (double)Stopwatch.GetTimestamp() / Stopwatch.Frequency;
                dt = time - previousTime;
                previousTime = time;
                
                space.update((float)dt);

                Thread.Sleep(0); //Give other threads a chance to execute
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
